/*
***************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2010 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/



#include "signalcurve.h"


SignalCurve::SignalCurve(QWidget *parent) : QWidget(parent)
{
  setAttribute(Qt::WA_OpaquePaintEvent);

  SignalColor = Qt::blue;
  tracewidth = 0;
  BackgroundColor = Qt::gray;
  RasterColor = Qt::darkGray;
  BorderColor = Qt::lightGray;
  RulerColor = Qt::black;
  TextColor = Qt::black;

  dbuf = NULL;
  fbuf = NULL;
  h_rastersize = 10;
  v_rastersize = 10;
  bordersize = 60;
  drawHruler = 1;
  drawVruler = 1;
  h_ruler_startvalue = 0.0;
  h_ruler_endvalue = 100.0;
  h_ruler_precision = 2;
  drawcurve_before_raster = 0;
  h_label[0] = 0;
  v_label[0] = 0;
  upperlabel[0] = 0;
  lowerlabel[0] = 0;
  max_value = 100.0;
  min_value = -100.0;
}


void SignalCurve::paintEvent(QPaintEvent *)
{
  int i, w, h,
      ruler_divisor,
      value_skip,
      precision;

  double v_sens,
         offset,
         h_step,
         rasterstep,
         value;

  QString q_str;


  w = width();
  h = height();

  QPainter painter(this);
  painter.fillRect(0, 0, w, h, BorderColor);

  if(w < ((bordersize * 2) + 5))  return;

  if(h < ((bordersize * 2) + 5))  return;

/////////////////////////////////// draw the rulers ///////////////////////////////////////////

  painter.setFont(QFont("Arial", 8));

  if((drawHruler) && (h_rastersize > 1) && (bordersize > 19))
  {
    painter.setPen(RulerColor);

    painter.drawLine(bordersize, h - bordersize + 5, w - bordersize, h - bordersize + 5);

    ruler_divisor = 10;

    rasterstep = (double)(w - (bordersize * 2)) / (h_rastersize * ruler_divisor);

    if(rasterstep < 7.0)
    {
      ruler_divisor = 5;

      rasterstep = (double)(w - (bordersize * 2)) / (h_rastersize * ruler_divisor);
    }

    if(rasterstep < 4.0)
    {
      ruler_divisor = 2;

      rasterstep = (double)(w - (bordersize * 2)) / (h_rastersize * ruler_divisor);
    }

    for(i = 0; i < ((h_rastersize * ruler_divisor) + 1); i++)
    {
      if(i % ruler_divisor)  painter.drawLine((i * rasterstep) + bordersize , h - bordersize + 5, (i * rasterstep) + bordersize, h - bordersize + 5 + 4);
    }

    rasterstep = (double)(w - (bordersize * 2)) / h_rastersize;

    value_skip = 1;

    if(rasterstep < 60)  value_skip = 2;
    if(rasterstep < 30)  value_skip = 4;
    if(rasterstep < 15)  value_skip = 8;
    if((w - (bordersize * 2)) < 120)  value_skip = h_rastersize;

    for(i = 0; i < (h_rastersize + 1); i++)
    {
      painter.drawLine((i * rasterstep) + bordersize , h - bordersize + 5, (i * rasterstep) + bordersize, h - bordersize + 5 + 10);

      if(!(i % value_skip))
      {
        value = h_ruler_startvalue - (((h_ruler_startvalue - h_ruler_endvalue) / h_rastersize) * i);

        q_str.setNum(value, 'f', h_ruler_precision);

        if(i == h_rastersize)
        {
          if(h_label[0] != 0)
          {
            q_str.append(" ");
            q_str.append(h_label);
          }
        }

        painter.drawText(bordersize + (i * rasterstep) - 30,  h - bordersize + 18, 60, 16, Qt::AlignCenter | Qt::TextSingleLine, q_str);
      }
    }
  }

  if((drawVruler) && (v_rastersize > 1) && (bordersize > 29))
  {
    painter.setPen(RulerColor);

    painter.drawLine(bordersize - 5, bordersize, bordersize - 5, h - bordersize);

    ruler_divisor = 10;

    rasterstep = (double)(h - (bordersize * 2)) / (v_rastersize * ruler_divisor);

    if(rasterstep < 7.0)
    {
      ruler_divisor = 5;

      rasterstep = (double)(h - (bordersize * 2)) / (v_rastersize * ruler_divisor);
    }

    if(rasterstep < 4.0)
    {
      ruler_divisor = 2;

      rasterstep = (double)(h - (bordersize * 2)) / (v_rastersize * ruler_divisor);
    }

    for(i = 0; i < ((v_rastersize * ruler_divisor) + 1); i++)
    {
      if(i % ruler_divisor)  painter.drawLine(bordersize - 5, bordersize + (i * rasterstep), bordersize - 5 - 4, bordersize + (i * rasterstep));
    }

    rasterstep = (double)(h - (bordersize * 2)) / v_rastersize;

    value_skip = 1;

    if(rasterstep < 40)  value_skip = 2;
    if(rasterstep < 20)  value_skip = 4;
    if(rasterstep < 10)  value_skip = 8;
    if((h - (bordersize * 2)) < 80)  value_skip = v_rastersize;

    for(i = 0; i < (v_rastersize + 1); i++)
    {
      painter.drawLine(bordersize - 5, bordersize + (i * rasterstep), bordersize - 5 - 10, bordersize + (i * rasterstep));

      if(!(i % value_skip))
      {
        value = max_value - (((max_value - min_value) / v_rastersize) * i);
        precision = 0;
        if(max_value < 1000.0)
        {
          precision = 1;
        }
        if(max_value < 100.0)
        {
          precision = 2;
        }
        if(max_value < 10.0)
        {
          precision = 3;
        }
        if(max_value < 1.0)
        {
          precision = 4;
        }
        q_str.setNum(value, 'f', precision);
        painter.drawText(3, bordersize + (i * rasterstep) - 8, 40, 16, Qt::AlignRight | Qt::AlignVCenter | Qt::TextSingleLine, q_str);
      }
    }
  }

/////////////////////////////////// draw the labels ///////////////////////////////////////////

  painter.setPen(TextColor);
  painter.setFont(QFont("Arial", 8));

  if(v_label[0] != 0)
  {
    painter.drawText(15, 30, 80, 16, Qt::AlignCenter | Qt::TextSingleLine, v_label);
  }

  if(upperlabel[0] != 0)
  {
    painter.drawText(w / 2 - 80, 20, 160, 16, Qt::AlignCenter | Qt::TextSingleLine, upperlabel);
  }

  if(lowerlabel[0] != 0)
  {
    painter.drawText(w / 2 - 80, h - 20, 160, 16, Qt::AlignCenter | Qt::TextSingleLine, lowerlabel);
  }

/////////////////////////////////// translate coordinates, draw and fill a rectangle ///////////////////////////////////////////

  painter.translate(QPoint(bordersize, bordersize));

  w -= (bordersize * 2);

  h -= (bordersize * 2);

  painter.fillRect(0, 0, w, h, BackgroundColor);

  painter.setClipping(TRUE);
  painter.setClipRegion(QRegion(0, 0, w, h), Qt::ReplaceClip);

/////////////////////////////////// draw the curve ///////////////////////////////////////////

  if(drawcurve_before_raster)
  {
    if((dbuf == NULL) && (fbuf == NULL)) return;

    if(max_value == min_value)  return;

    if(bufsize < 2)  return;

    offset = (-(max_value));

    v_sens = (-(h / (max_value - min_value)));

    h_step = (double)w / (double)(bufsize - 1);

    painter.setPen(QPen(QBrush(SignalColor, Qt::SolidPattern), tracewidth, Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin));

    if(dbuf)
    {
      for(i = 0; i < (bufsize - 1); i++)
      {
        painter.drawLine(i * h_step, (dbuf[i] + offset) * v_sens, (i + 1) * h_step, (dbuf[i + 1] + offset) * v_sens);
      }
    }

    if(fbuf)
    {
      for(i = 0; i < (bufsize - 1); i++)
      {
        painter.drawLine(i * h_step, (fbuf[i] + offset) * v_sens, (i + 1) * h_step, (fbuf[i + 1] + offset) * v_sens);
      }
    }
  }

/////////////////////////////////// draw the rasters ///////////////////////////////////////////

  painter.setPen(RasterColor);

  painter.drawRect (0, 0, w - 1, h - 1);

  if(h_rastersize > 1)
  {
    rasterstep = (double)w / h_rastersize;

    for(i = 1; i < h_rastersize; i++)
    {
      painter.drawLine(i * rasterstep, 0, i * rasterstep, h);
    }
  }

  if(v_rastersize > 1)
  {
    rasterstep = (double)h / v_rastersize;

    for(i = 1; i < v_rastersize; i++)
    {
      painter.drawLine(0, i * rasterstep, w, i * rasterstep);
    }
  }

/////////////////////////////////// draw the curve ///////////////////////////////////////////

  if(!drawcurve_before_raster)
  {
    if((dbuf == NULL) && (fbuf == NULL)) return;

    if(max_value == min_value)  return;

    if(bufsize < 2)  return;

    offset = (-(max_value));

    v_sens = (-(h / (max_value - min_value)));

    h_step = (double)w / (double)(bufsize - 1);

    painter.setPen(QPen(QBrush(SignalColor, Qt::SolidPattern), tracewidth, Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin));

    if(dbuf)
    {
      for(i = 0; i < (bufsize - 1); i++)
      {
        painter.drawLine(i * h_step, (dbuf[i] + offset) * v_sens, (i + 1) * h_step, (dbuf[i + 1] + offset) * v_sens);
      }
    }

    if(fbuf)
    {
      for(i = 0; i < (bufsize - 1); i++)
      {
        painter.drawLine(i * h_step, (fbuf[i] + offset) * v_sens, (i + 1) * h_step, (fbuf[i + 1] + offset) * v_sens);
      }
    }
  }
}


void SignalCurve::drawCurve(double *samplebuf, int size, double max_val, double min_val)
{
  dbuf = samplebuf;

  fbuf = NULL;

  bufsize = size;

  max_value = max_val;

  min_value = min_val;

  update();
}


void SignalCurve::drawCurve(float *samplebuf, int size, double max_val, double min_val)
{
  fbuf = samplebuf;

  dbuf = NULL;

  bufsize = size;

  max_value = max_val;

  min_value = min_val;

  update();
}


void SignalCurve::setH_RulerValues(double start, double end, int precision)
{
  h_ruler_startvalue = start;
  h_ruler_endvalue = end;
  h_ruler_precision = precision;

  update();
}


void SignalCurve::setSignalColor(QColor newColor)
{
  SignalColor = newColor;
  update();
}


void SignalCurve::setTraceWidth(int width)
{
  tracewidth = width;
  if(tracewidth < 0)  tracewidth = 0;
  update();
}


void SignalCurve::setBackgroundColor(QColor newColor)
{
  BackgroundColor = newColor;
  update();
}


void SignalCurve::setRasterColor(QColor newColor)
{
  RasterColor = newColor;
  update();
}


void SignalCurve::setBorderColor(QColor newColor)
{
  BorderColor = newColor;
  update();
}


void SignalCurve::setTextColor(QColor newColor)
{
  TextColor = newColor;
  update();
}


void SignalCurve::setH_RasterSize(int newsize)
{
  h_rastersize = newsize;
  update();
}


void SignalCurve::setV_RasterSize(int newsize)
{
  v_rastersize = newsize;
  update();
}


void SignalCurve::setBorderSize(int newsize)
{
  bordersize = newsize;
  if(bordersize < 0)  bordersize = 0;
  update();
}


void SignalCurve::setH_label(const char *str)
{
  strncpy(h_label, str, 32);
  h_label[31] = 0;
  update();
}


void SignalCurve::setV_label(const char *str)
{
  strncpy(v_label, str, 8);
  v_label[7] = 0;
  update();
}


void SignalCurve::setUpperLabel(const char *str)
{
  strncpy(upperlabel, str, 64);
  upperlabel[63] = 0;
  update();
}


void SignalCurve::setLowerLabel(const char *str)
{
  strncpy(lowerlabel, str, 64);
  lowerlabel[63] = 0;
  update();
}









