/*
***************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2010 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/



#include "spectrumanalyzer.h"



UI_FreqSpectrumWindow::UI_FreqSpectrumWindow(struct signalcompblock *signal_comp, char *view_buf, QWidget *parent)
{
  char str[128];

  buf1 = NULL;
  buf2 = NULL;
  buf3 = NULL;

  samples = 0;

  parentwindow = parent;

  viewbuf = view_buf;

  signalcomp = signal_comp;

  strcpy(signallabel, signalcomp->signallabel);

  strcpy(physdimension, signalcomp->physdimension);

  sqrt_powerspectrum = 0;

  samples = signalcomp->samples_on_screen;

  samplefreq = (double)signalcomp->edfhdr->edfparam[signalcomp->edfsignal[0]].smp_per_record / ((double)signalcomp->edfhdr->long_data_record_duration / TIME_DIMENSION);

  startsample = 0;

  freqstep = samplefreq / (double)samples;

  steps = ((double)samplefreq / 2.0) / freqstep;

  SpectrumDialog = new QDialog();
  SpectrumDialog->setModal(TRUE);
  SpectrumDialog->setAttribute(Qt::WA_DeleteOnClose, TRUE);
  SpectrumDialog->setMinimumSize(QSize(640, 480));
  SpectrumDialog->setSizeGripEnabled(TRUE);
  SpectrumDialog->setWindowFlags(Qt::WindowMinMaxButtonsHint | SpectrumDialog->windowFlags());
  if(sqrt_powerspectrum)
  {
    sprintf(str, "Amplitude Spectrum %s", signallabel);
  }
  else
  {
    sprintf(str, "Power Spectrum %s", signallabel);
  }
  SpectrumDialog->setWindowTitle(str);

  curve1 = new SignalCurve;
  curve1->setSignalColor(Qt::green);
  curve1->setBackgroundColor(Qt::black);
  curve1->setRasterColor(Qt::gray);
  curve1->setTraceWidth(0);
  curve1->setH_RasterSize(10);
  curve1->setV_RasterSize(10);
  curve1->setH_label("Hz");
  curve1->setLowerLabel("Frequency");
  curve1->setH_RulerValues(0, (double)samplefreq / 2.0, 0);
  if(sqrt_powerspectrum)
  {
    curve1->setV_label(physdimension);
  }
  else
  {
    curve1->setV_label("Power");
  }

  amplitudeSlider = new QSlider;
  amplitudeSlider->setOrientation(Qt::Vertical);
  amplitudeSlider->setMinimum(1);
  amplitudeSlider->setMaximum(1000);
  amplitudeSlider->setValue(1000);
  amplitudeSlider->setInvertedAppearance(TRUE);
  amplitudeSlider->setMinimumSize(15, 280);

  amplitudeLabel = new QLabel;
  amplitudeLabel->setText("Amplitude");
  amplitudeLabel->setMinimumSize(100, 15);
  amplitudeLabel->setAlignment(Qt::AlignHCenter);

  sqrtButton = new QPushButton;
  sqrtButton->setMinimumSize(50, 20);
  sqrtButton->setText("Sqrt/Power");

  vlayout2 = new QVBoxLayout;
  vlayout2->setSpacing(20);
  vlayout2->addStretch(100);
  vlayout2->addWidget(amplitudeLabel, 0, Qt::AlignHCenter);
  vlayout2->addWidget(amplitudeSlider, 0, Qt::AlignHCenter);
  vlayout2->addWidget(sqrtButton);

  spanSlider = new QSlider;
  spanSlider->setOrientation(Qt::Horizontal);
  spanSlider->setMinimum(10);
  spanSlider->setMaximum(1000);
  spanSlider->setValue(1000);
  spanSlider->setMinimumSize(500, 15);

  spanLabel = new QLabel;
  spanLabel->setText("Span");
  spanLabel->setMinimumSize(100, 15);
  spanLabel->setAlignment(Qt::AlignHCenter);

  centerSlider = new QSlider;
  centerSlider->setOrientation(Qt::Horizontal);
  centerSlider->setMinimum(0);
  centerSlider->setMaximum(1000);
  centerSlider->setValue(0);
  centerSlider->setMinimumSize(500, 15);

  centerLabel = new QLabel;
  centerLabel->setText("Center");
  centerLabel->setMinimumSize(100, 15);
  centerLabel->setAlignment(Qt::AlignHCenter);

  hlayout1 = new QHBoxLayout;
  hlayout1->setSpacing(20);
  hlayout1->addLayout(vlayout2);
  hlayout1->addWidget(curve1, 100);

  hlayout2 = new QHBoxLayout;
  hlayout2->setSpacing(20);
  hlayout2->addWidget(spanLabel);
  hlayout2->addWidget(spanSlider);
  hlayout2->addStretch(100);

  hlayout3 = new QHBoxLayout;
  hlayout3->setSpacing(20);
  hlayout3->addWidget(centerLabel);
  hlayout3->addWidget(centerSlider);
  hlayout3->addStretch(100);

  vlayout1 = new QVBoxLayout;
  vlayout1->setSpacing(20);
  vlayout1->addLayout(hlayout1);
  vlayout1->addLayout(hlayout2);
  vlayout1->addLayout(hlayout3);

  SpectrumDialog->setLayout(vlayout1);

  t1 = new QTimer;
  t1->setSingleShot(TRUE);
  t1->start(100);

  QObject::connect(t1,              SIGNAL(timeout()),         this, SLOT(timer1()));
  QObject::connect(amplitudeSlider, SIGNAL(valueChanged(int)), this, SLOT(sliderMoved(int)));
  QObject::connect(spanSlider,      SIGNAL(valueChanged(int)), this, SLOT(sliderMoved(int)));
  QObject::connect(centerSlider,    SIGNAL(valueChanged(int)), this, SLOT(sliderMoved(int)));
  QObject::connect(sqrtButton,      SIGNAL(clicked()),         this, SLOT(sqrtButtonClicked()));

  SpectrumDialog->exec();
}


void UI_FreqSpectrumWindow::sqrtButtonClicked()
{
  char str[128];

  if(sqrt_powerspectrum)
  {
    sqrt_powerspectrum = 0;

    sprintf(str, "Power Spectrum %s", signallabel);

    curve1->setV_label("Power");
  }
  else
  {
    sqrt_powerspectrum = 1;

    sprintf(str, "Amplitude Spectrum %s", signallabel);

    curve1->setV_label(physdimension);
  }

  SpectrumDialog->setWindowTitle(str);

  sliderMoved(0);
}


void UI_FreqSpectrumWindow::sliderMoved(int)
{
  int startstep,
      stopstep,
      precision,
      spanstep;

  double max_freq,
         start_freq;


  spanstep = spanSlider->value() * steps / 1000;

  startstep = centerSlider->value() * (steps - spanstep) / 1000;

  stopstep = startstep + spanstep;

  if(sqrt_powerspectrum)
  {
    curve1->drawCurve(buf3 + startstep, stopstep - startstep, (maxvalue_sqrt * 1.05 * (double)amplitudeSlider->value()) / 1000.0, 0.0);
  }
  else
  {
    curve1->drawCurve(buf2 + startstep, stopstep - startstep, (maxvalue * 1.05 * (double)amplitudeSlider->value()) / 1000.0, 0.0);
  }

  max_freq = ((double)samplefreq / 2.0) * stopstep / steps;

  precision = 0;
  if(max_freq < 1000.0)
  {
    precision = 1;
  }
  if(max_freq < 100.0)
  {
    precision = 2;
  }
  if(max_freq < 10.0)
  {
    precision = 3;
  }
  if(max_freq < 1.0)
  {
    precision = 4;
  }

  start_freq = ((double)samplefreq / 2.0) * startstep / steps;

  curve1->setH_RulerValues(start_freq, max_freq, precision);

  centerLabel->setText(QString::number(start_freq + ((max_freq - start_freq) / 2.0), 'f', precision).append(" Hz").prepend("Center "));

  spanLabel->setText(QString::number(max_freq - start_freq, 'f', precision).append(" Hz").prepend("Span "));
}


void UI_FreqSpectrumWindow::timer1()
{
  int i, j, k,
      dig_value=0,
      temp=0;

  long long s, s2;

  char str[128];

  union {
          unsigned int one;
          signed int one_signed;
          unsigned short two[2];
          signed short two_signed[2];
          unsigned char four[4];
        } var;


  if(viewbuf == NULL)
  {
    return;
  }

  samples = signalcomp->samples_on_screen;

  if(samples < 32)
  {
    UI_Messagewindow popupmessage("Warning", "Not enough samples, please increase the displaytime.");
    SpectrumDialog->close();
    return;
  }

  if(samples > 100000)
  {
    UI_Messagewindow popupmessage("Warning", "Too many samples, please decrease the displaytime.");
    SpectrumDialog->close();
    return;
  }

  samplefreq = (double)signalcomp->edfhdr->edfparam[signalcomp->edfsignal[0]].smp_per_record / ((double)signalcomp->edfhdr->long_data_record_duration / TIME_DIMENSION);

  startsample = 0;

  freqstep = samplefreq / (double)samples;

  steps = ((double)samplefreq / 2.0) / freqstep;

  buf1 = (double *)malloc(sizeof(double) * signalcomp->samples_on_screen);
  if(buf1 == NULL)
  {
    UI_Messagewindow popupmessage("Error", "Malloc error (buf1)");
    return;
  }

  buf2 = (double *)malloc(sizeof(double) * steps);
  if(buf2 == NULL)
  {
    UI_Messagewindow popupmessage("Error", "Malloc error (buf2)");
    free(buf1);
    buf1 = NULL;
    return;
  }

  buf3 = (double *)malloc(sizeof(double) * steps);
  if(buf3 == NULL)
  {
    UI_Messagewindow popupmessage("Error", "Malloc error (buf3)");
    free(buf1);
    free(buf2);
    buf1 = NULL;
    buf2 = NULL;
    return;
  }

  samples = 0;

  for(s=signalcomp->sample_start; s<signalcomp->samples_on_screen; s++)
  {
    if(s>signalcomp->sample_stop)  break;

    dig_value = 0;
    s2 = s + signalcomp->sample_timeoffset - signalcomp->sample_start;

    for(j=0; j<signalcomp->num_of_signals; j++)
    {
      if(signalcomp->edfhdr->bdf)
      {
        var.two[0] = *((unsigned short *)(
          viewbuf
          + signalcomp->viewbufoffset
          + (signalcomp->edfhdr->recordsize * (s2 / signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].smp_per_record))
          + signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].buf_offset
          + ((s2 % signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].smp_per_record) * 3)));

        var.four[2] = *((unsigned char *)(
          viewbuf
          + signalcomp->viewbufoffset
          + (signalcomp->edfhdr->recordsize * (s2 / signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].smp_per_record))
          + signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].buf_offset
          + ((s2 % signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].smp_per_record) * 3)
          + 2));

        if(var.four[2]&0x80)
        {
          var.four[3] = 0xff;
        }
        else
        {
          var.four[3] = 0x00;
        }

        temp = var.one_signed;
      }

      if(signalcomp->edfhdr->edf)
      {
        temp = *(((short *)(
          viewbuf
          + signalcomp->viewbufoffset
          + (signalcomp->edfhdr->recordsize * (s2 / signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].smp_per_record))
          + signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].buf_offset))
          + (s2 % signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].smp_per_record));
      }

      temp += signalcomp->edfhdr->edfparam[signalcomp->edfsignal[j]].offset;
      temp *= signalcomp->factor[j];

      dig_value += temp;
    }

    for(k=0; k<signalcomp->filter_cnt; k++)
    {
      dig_value = first_order_filter(dig_value, signalcomp->filter[k]);
    }

    for(k=0; k<signalcomp->fidfilter_cnt; k++)
    {
      dig_value = signalcomp->fidfuncp[k](signalcomp->fidbuf[k], dig_value);
    }

    if(s>=signalcomp->sample_start)
    {
      if(samples == signalcomp->samples_on_screen)
      {
        break;
      }

      buf1[samples++] = dig_value * signalcomp->edfhdr->edfparam[signalcomp->edfsignal[0]].bitvalue;
    }
  }

  samplefreq = (double)signalcomp->edfhdr->edfparam[signalcomp->edfsignal[0]].smp_per_record / ((double)signalcomp->edfhdr->long_data_record_duration / TIME_DIMENSION);

  startsample = 0;

  freqstep = samplefreq / (double)samples;

  steps = ((double)samplefreq / 2.0) / freqstep;

  maxvalue = 0.0001;
  maxvalue_sqrt = 0.0001;

  QProgressDialog progress("Computing...", "Cancel", 0, steps, this);

  progress.setWindowModality(Qt::WindowModal);
  progress.setMinimumDuration(1000);

  for(i=0; i<steps; i++)
  {
    buf2[i] = fourier_transform_get_power_double(i, buf1, samples, samplefreq);

    if(buf2[i] > maxvalue)
    {
      maxvalue = buf2[i];
    }

    buf3[i] = sqrt(buf2[i]);

    if(buf3[i] > maxvalue_sqrt)
    {
      maxvalue_sqrt = buf3[i];
    }

    if(!(i % 100))
    {
      progress.setValue(i);

      qApp->processEvents();

      if(progress.wasCanceled())
      {
        SpectrumDialog->close();
        return;
      }
    }
  }

  if(buf1 != NULL)
  {
    free(buf1);
  }

  sprintf(str, "DFT resolution: %f Hz", freqstep);

  remove_trailing_zeros(str);

  curve1->setUpperLabel(str);

  sliderMoved(0);

  progress.setValue(steps);
}



UI_FreqSpectrumWindow::~UI_FreqSpectrumWindow()
{
  if(buf2 != NULL)
  {
    free(buf2);
  }

  if(buf3 != NULL)
  {
    free(buf3);
  }
}




















