/*
***************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2010 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/



#ifndef SPECTRUMANALYZER_H
#define SPECTRUMANALYZER_H



#include <QApplication>
#include <QDialog>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QTimer>
#include <QSlider>
#include <QLabel>
#include <QProgressDialog>
#include <QPushButton>

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "global.h"
#include "signalcurve.h"
#include "fourier.h"
#include "popup_messagewindow.h"
#include "utils.h"






class UI_FreqSpectrumWindow : public QWidget
{
  Q_OBJECT

public:
  UI_FreqSpectrumWindow(struct signalcompblock *, char *, QWidget *parent=0);

  ~UI_FreqSpectrumWindow();

private:

  QWidget *parentwindow;

  QDialog *SpectrumDialog;

  QHBoxLayout *hlayout1,
              *hlayout2,
              *hlayout3;

  QVBoxLayout *vlayout1,
              *vlayout2;

  SignalCurve *curve1;

  QTimer *t1;

  QSlider *amplitudeSlider,
          *spanSlider,
          *centerSlider;

  QLabel *spanLabel,
         *centerLabel,
         *amplitudeLabel;

  QPushButton *sqrtButton;

  int startsample,
      samples,
      steps,
      sqrt_powerspectrum;

  double samplefreq,
         freqstep,
         maxvalue,
         maxvalue_sqrt,
         *buf1,
         *buf2,
         *buf3;

  char *viewbuf,
       signallabel[512],
       physdimension[9];

  struct signalcompblock *signalcomp;


private slots:

void timer1();
void sliderMoved(int);
void sqrtButtonClicked();

};


#endif



















